#include "utils.h"
#include "support.h"
#include "interface.h"
#include "tpinterface.h"

gchar* elapsed(double);

const gchar* const browser_list[][2] = {
    {"firefox", "-new-window"},
    {"konqueror", NULL},
    {"mozilla", "-new-window"},
    {"epiphany", "-new-window"},
    {NULL, NULL}
};

BrowserInfo my_browser;

BrowserInfo get_browser();

void flush_display(void)
{
    GdkDisplay *display;

    display = gdk_display_get_default();
    XFlush(GDK_DISPLAY_XDISPLAY(display));
}

gchar* elapsed(double times)
{
    gchar* elap;
    short sec, min, hour;
    long ticks = (long) times;

    elap = g_strdup("00:00:00");
    if (ticks > 0)
    {
	hour = (ticks % 3600)? ticks / 3600 : 0;
	if (hour)
	    ticks %= 3600;
	if (ticks % 3600)
	    min = (ticks % 60)? ticks / 60 : 0;
	else
	{
	    hour++;
	    min = 0;
	    ticks = 0;
	}
	if (min)
	{
	    ticks %= 60;
	    if (ticks % 60)
		sec = ticks;
	    else
	    {
		min++;
		sec = 0;
	    }
	}
	else
	{
	    sec = ticks % 60;
	    if (!sec && ticks)
		min++;
	}
	elap = g_strdup_printf("%02d:%02d:%02d", hour, min, sec);
    }
    return elap;
}

void do_print(PrintInfo* print_info, gboolean preview)
{
    FILE*	prt = NULL;
    int		result;
    gchar*	format = NULL;
    gchar*	teststart = NULL;
    gchar*	duration = NULL;
    DateInfo*	date = NULL;
    GtkWidget*	dialog = NULL;
    GtkWidget*	label = NULL;

    g_return_if_fail(print_info != NULL);
    date = get_date(SHORT, 0);
    format = g_strdup_printf("\t\n%s - %s\n", date->date, date->time);
    duration = elapsed(difftime(print_info->stop_time, print_info->start_time));
    date = get_date(SHORT, print_info->start_time);
    teststart = g_strdup_printf("%s %s", date->date, date->time);
    format = g_strconcat(format, _("\n\t\t\t\tTPTEST Session Report\n\n"), NULL);
    format = g_strdup_printf(_("%s\t\tTest Started:\t\t%s\n\t\tDuration:\t\t%s\n\t\tTest Server:\t\t%s\n\t\tBest UDP Send Rate:\t%s\n\t\tBest UDP Receive Rate:\t%s\n\t\tBest TCP Send Rate:\t%s\n\t\tBest TCP Receive Rate:\t%s\n\n\t\tSend Efficiency:\t%s\n\t\tReceive Efficiency:\t%s\n"),
	    format, teststart, duration, print_info->server,
	    print_info->udp_best_send, print_info->udp_best_receive,
	    print_info->tcp_best_send, print_info->tcp_best_receive,
	    print_info->send_best_load, print_info->receive_best_load);
    if (DEBUG)
	fprintf(stderr, "%s\n", format);
    if (preview)
    {
	g_return_if_fail(format != NULL);

	dialog = create_result_dialog();
	label = lookup_widget(dialog, "label18");
		
	gtk_label_set_text((GtkLabel *) label, format);
	int h,w;
	gtk_window_get_size((GtkWindow *) dialog, &w, &h);
	gtk_window_resize((GtkWindow *) dialog, 400, 200);
	gtk_window_get_size((GtkWindow *) dialog, &w, &h);
	gtk_window_set_modal((GtkWindow *) dialog, TRUE);
	gtk_widget_show(dialog);
	flush_display();
    }
    else
    {
	if ((prt = popen("lpr -T TPTEST -h -p", "w")) == NULL)
	    g_critical(_("Printing was not allowed"));
	fprintf(prt, format);
	fflush(prt);
	result = pclose(prt);
	if (!WIFEXITED(result))
	    g_warning(_("Abnormal exit"));
    }
    g_free(format);
    g_free(teststart);
    g_free(duration);
    g_free(date);
}

BrowserInfo get_browser()
{
#define MAX_TOKENS 1000

    size_t  i;
    gchar*   env_path = NULL;
    gchar*  file = NULL;
    gchar** path = NULL;
    gchar** head = NULL;
    
    my_browser.name = NULL;
    my_browser.option = NULL;
    head = g_malloc(sizeof(gchar*));

    if ((env_path = getenv("PATH")) == NULL)
	g_error(_("Could not get PATH from environment"));
    path = g_strsplit(env_path, ":", MAX_TOKENS);
    head = path;

    if (path == NULL)
	g_error(_("PATH is empty or not defined"));

    for (i = 0; *browser_list[i] != NULL && !my_browser.name; i++)
    {
	if (DEBUG)
	    fprintf(stderr, "Looking for: %s\n", *browser_list[i]);

	while (*path++)
	{
	    file = g_strdup_printf("%s/%s", *path, *browser_list[i]);
	    if (access(file, F_OK | R_OK | X_OK))
		/* The above access modes was not allowed or
		 * the file could not be found - X_OK must be
		 * on the holde path tree
		 */
		continue;
	    else
	    {
		my_browser.name = g_strdup(file);
		my_browser.option = g_strdup(browser_list[i][1]);
		if (DEBUG)
		{
		    fprintf(stderr, "Browser found: %s\n", my_browser.name);
		    fprintf(stderr, "Using option: %s\n", my_browser.option);
		}
		break;
	    }
	}
    }
    g_free(file);
    g_strfreev(head);
    return my_browser;
}

void activate_url(GtkAboutDialog *dialog, const gchar *link, gpointer data)
{
    pid_t pid;

    g_return_if_fail(my_browser.name != NULL);
    g_return_if_fail(link != NULL);

    pid = fork();
    if (pid == 0)
    {
	execlp(my_browser.name, my_browser.name, my_browser.option, link , NULL);
	exit(0);
    }
}

DateInfo* get_date(DATETYPE type, time_t myTime)
{
    struct tm* date = NULL;
    DateInfo* date_info = NULL;
    size_t n;
    time_t time_ticks;
    gchar *dateformat = NULL, *timeformat = NULL;
    gchar buf[BUFSIZ];

    dateformat = timeformat = NULL;
    switch (type) {
	case SHORT:
	    dateformat = g_strdup("%d/%m/%Y");
	    timeformat = g_strdup("%H:%M:%U");
	    break;
	case LONG:
	    dateformat = g_strdup("%A %d. %B %Y");
	    timeformat = g_strdup("%H:%M:%U");
	    break;
    }

    date_info = g_malloc(sizeof(DateInfo));
    time_ticks = (myTime)? myTime: time(NULL);
    date = localtime(&time_ticks);
    g_return_val_if_fail(date != NULL, date_info);
    
    n = strftime(buf, BUFSIZ, dateformat, date);
    date_info->date = g_strdup(buf);
    if (n == BUFSIZ)
    {
	date_info->date = NULL;
	g_error(_("Formatting time string exceeds maximum buffer size"));
    }
    n = strftime(buf, BUFSIZ, timeformat, date);
    date_info->time = g_strdup(buf);
    if (n == BUFSIZ)
    {
	date_info->time = NULL;
	g_error(_("Formatting time string exceeds maximum buffer size"));
    }
    g_free(timeformat);
    g_free(dateformat);
    return date_info;
}

GtkWidget* messagebox(
	const gchar* text, const gchar* subtext,
	GtkMessageType msg, GtkButtonsType btn)
{
    GtkWidget* dialog = NULL;

    dialog = gtk_message_dialog_new(
	    (GtkWindow *) mainwindow,
	    GTK_DIALOG_DESTROY_WITH_PARENT,
	    msg, btn, text);
    gtk_message_dialog_format_secondary_text(
	(GtkMessageDialog *) dialog, subtext);

    return dialog;
}

void init(void)
{
    GtkWidget*	master_server = NULL;
    GtkWidget*	test_server = NULL;
    gchar*	server = NULL;
    gchar*	m_srv = MASTERSERVER;
    gchar*	t_srv = TESTSERVER;

    master_server = lookup_widget(mainwindow, "master_entry");
    test_server = lookup_widget(mainwindow, "server_entry");

    if (m_srv)
    {
	server = g_strdup_printf("%s:%d", m_srv, (int) MASTERSERVERPORT);
	gtk_entry_set_text((GtkEntry *) master_server, server);
    }
    else
	gtk_entry_set_text((GtkEntry *) master_server, _("NONE"));
    if (t_srv)
    {
	server = g_strdup_printf("%s:%d", t_srv, (int) TESTSERVERPORT);
	gtk_entry_set_text((GtkEntry *) test_server, server);
    }
}

